function love.load()
	require("camera")
	require("items")
	
	love.physics.setMeter(64) --Hhe eines Meters (px), Orientierungsgre fr alle physikalischen Krfte
	gravitation = 2
	speed = 800
	maxSpeed = 200
	size = 650
	score = 0
	pause = true
	
	decoder = love.sound.newSoundData("engine.mp3")
	engine = love.audio.newSource(decoder)
	engine:setLooping(true)
	engine:play()
	engine:setVolume(0)
	decoder = love.sound.newSoundData("darkLoop.mp3")
	loop = love.audio.newSource(decoder)
	loop:setLooping(true)
	loop:play()
	loop:setVolume(0)
	decoder = love.sound.newSoundData("chime.wav")
	chime = love.audio.newSource(decoder)
	chime:setLooping(false)
	chime:stop()
	decoder = love.sound.newSoundData("fuel.mp3")
	fuelSound = love.audio.newSource(decoder)
	fuelSound:setLooping(false)
	fuelSound:stop()
	
	particle = love.graphics.newImage("particle.png")
	galaxy = love.graphics.newImage("galaxy.png")
	world = love.physics.newWorld(0, 0, true) --Horizontale Gravitation, Vertikale Gravitation, schlafende Bodies erlaubt
	camera.layers = {}
	
	objects = {} --Objektetabelle
	
	ship = {}
	ship.body = love.physics.newBody(world, size/2, size/2, "dynamic")
	ship.body:setMass(15)
	ship.body:setLinearDamping(0.3)
	ship.shape = love.physics.newCircleShape(10)
	ship.fixture = love.physics.newFixture(ship.body, ship.shape, 1)
	ship.fixture:setRestitution(0.9)
	ship.image = love.graphics.newImage("ship.png")
	ship.rotation = 0
	ship.alive = true
	ship.motor = love.graphics.newParticleSystem(particle, 25)
	ship.motor:setEmissionRate(20)
	ship.motor:setLifetime(1)
	ship.motor:setParticleLife(1)
	ship.motor:setPosition(0, 0)
	ship.motor:setDirection(1)
	ship.motor:setSpread(1)
	ship.motor:setSpeed(10, 20)
	ship.motor:setGravity(0)
	ship.motor:setSizeVariation(0.5)
	ship.motor:setRotation(0)
	ship.motor:setSpin(0)
	ship.motor:setSpinVariation(0)
	ship.motor:stop();
	
	hole = {}
	hole.body = love.physics.newBody(world, size/2, size-50/2)
	hole.shape = love.physics.newCircleShape(20)
	hole.fixture = love.physics.newFixture(hole.body, hole.shape);
	hole.radius = hole.shape:getRadius()
	
	camera:newLayer(0.05, function()
        love.graphics.setColor(200,200,200)
        love.graphics.draw(galaxy, -150, -150, 0, 1, 1)
        love.graphics.setColor(255, 255, 255)
    end)
	camera:newLayer(1, function()
        love.graphics.setColor(200,200,200, 20)
		local roundedPositionX = ship.body:getX() - ship.body:getX() % 150
		local minX = roundedPositionX - 500
		local maxX = roundedPositionX + 500
		local roundedPositionY = ship.body:getY() - ship.body:getY() % 150
		local minY = roundedPositionY - 500
		local maxY = roundedPositionY + 500
		for var = 1, 6, 1 do
			love.graphics.line(minX, roundedPositionY - 500 + 150 * var, maxX, roundedPositionY - 500 + 150 * var)
			love.graphics.line(roundedPositionX - 500 + 150 * var, minY, roundedPositionX - 500 + 150 * var, maxY)
		end
		
        love.graphics.setColor(255,0,0, 65)
		love.graphics.line(ship.body:getX(), ship.body:getY(), hole.body:getX(), hole.body:getY())
        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.draw(ship.motor, ship.body:getX() + math.cos(ship.rotation + math.rad(120)) * 13, ship.body:getY() + math.sin(ship.rotation + math.rad(120)) * 13)
		love.graphics.draw(ship.motor, ship.body:getX() + math.cos(ship.rotation + math.rad(60)) * 13, ship.body:getY() + math.sin(ship.rotation + math.rad(60)) * 13)
		love.graphics.draw(ship.image, ship.body:getX() + ship.image:getWidth() / 2 - 10, ship.body:getY(), ship.rotation, 1, 1, ship.image:getWidth() / 2, ship.image:getHeight() / 2 + 5)
		
		for _, v in ipairs(objects) do
			love.graphics.draw(v.image, v.body:getX(), v.body:getY(), v.body:getAngle(), 1, 1, v.image:getWidth() / 2, v.image:getHeight() / 2)
			if v.type == "shield" then
				love.graphics.setColor(100, 100, 255, 30)
				love.graphics.circle("fill", v.body:getX(), v.body:getY(), 100)
				love.graphics.setColor(50, 50, 200, 255)
				love.graphics.circle("line", v.body:getX(), v.body:getY(), 100)
				love.graphics.setColor(255, 255, 255, 255)
			end
		end
		
		love.graphics.setColor(0, 0, 0)
		love.graphics.circle("fill", hole.body:getX(), hole.body:getY(), hole.radius)
		love.graphics.setColor(200, 20, 100)
		love.graphics.circle("line", hole.body:getX(), hole.body:getY(), hole.radius)
        love.graphics.setColor(255, 255, 255)
    end)
	
	love.graphics.setBackgroundColor(0, 0, 0)
	love.graphics.setMode(size, size, false, true, 0)
	love.graphics.setIcon(love.graphics.newImage("gravity.png"))
	love.graphics.setCaption("Gravity")
end

function love.update(dt)
	if not pause then
		world:update(dt)
		
		hole.radius = hole.radius + dt * 5
		gravitation = gravitation + dt * 0.01
		
		camera.x = ship.body:getX() - size / 2
		camera.y = ship.body:getY() - size / 2
		
		local a, b = ship.body:getLinearVelocity()
		ship.motor:update(dt)
		
		zeroGravity = false
		for pos, v in ipairs(objects) do
			applyGravity(v.body)
			if math.sqrt( math.pow(hole.body:getX() - v.body:getX(), 2) + math.pow(hole.body:getY() - v.body:getY(), 2)) < hole.radius - 20 then
				v.body:destroy()
				v = nil
				table.remove(objects, pos)
			elseif ship.alive then
				if v.type == "shield" and math.sqrt( math.pow(ship.body:getX() - v.body:getX(), 2) + math.pow(ship.body:getY() - v.body:getY(), 2)) < 100 then
					zeroGravity = true
				end
				if v.type == "item" and math.sqrt( math.pow(ship.body:getX() - v.body:getX(), 2) + math.pow(ship.body:getY() - v.body:getY(), 2)) < 30 then
					if v.id == 12 then -- fuel
						speed = speed * 1.1
						maxSpeed = maxSpeed + 50
						fuelSound:rewind()
						fuelSound:play()
					elseif v.id == 13 then -- inverser
						gravitation = gravitation * 0.95
						hole.radius = hole.radius * 0.95
						fuelSound:rewind()
						fuelSound:play()
					else -- coin
						score = score + 1
						chime:rewind()
						chime:play()
					end
					v.body:destroy()
					table.remove(objects, pos)
				end
			end
		end
		if not zeroGravity then
			applyGravity(ship.body)
		end
		
		if math.random(500) > 494 then
			sign = math.random(2)
			if sign == 2 then
				sign = -1
			end
			quadrant = math.random(4)
			if quadrant == 1 then
				addItem(ship.body:getX() + size / 2 + 100, ship.body:getY() + math.random(size / 2) * sign)
			elseif quadrant == 2 then
				addItem(ship.body:getX() - size / 2 - 100, ship.body:getY() + math.random(size / 2) * sign)
			elseif quadrant == 3 then
				addItem(ship.body:getX() + math.random(size / 2) * sign, ship.body:getY() + size / 2 + 100)
			else                                                                          
				addItem(ship.body:getX() + math.random(size / 2) * sign, ship.body:getY() - size / 2 - 100)
			end
		end
		if (math.sqrt( math.pow(hole.body:getX() - ship.body:getX(), 2) + math.pow(hole.body:getY() - ship.body:getY(), 2)) + hole.radius) < 2000 then
			loop:setVolume( 1 - 0.0005 * math.sqrt( math.pow(hole.body:getX() - ship.body:getX(), 2) + math.pow(hole.body:getY() - ship.body:getY(), 2))  )
		end
		
		if math.sqrt( math.pow(hole.body:getX() - ship.body:getX(), 2) + math.pow(hole.body:getY() - ship.body:getY(), 2)) > hole.radius then
			if love.keyboard.isDown("right") then --Rechts: Schiff im Uhrzeigersinn drehen
				ship.rotation = ship.rotation + 2 * dt
				ship.motor:setDirection(ship.rotation + math.rad(90))
			end
			if love.keyboard.isDown("left") then --Links: Schiff gegen den Uhrzeigersinn drehen
				ship.rotation = ship.rotation - 2 * dt
				ship.motor:setDirection(ship.rotation + math.rad(90))
			end
			if love.keyboard.isDown("up") then --press the up arrow key to set the ball in the air
				local c = math.sqrt(math.pow(a, 2) + math.pow(b, 2))
				if c < maxSpeed then
					ship.body:applyForce(math.sin(ship.rotation) * speed * dt, math.cos(ship.rotation) * -1 * speed * dt)
				end
				ship.motor:start()
				if engine:getVolume() < 1 then
					if engine:getVolume() == 0 then
						engine:rewind()
					end
					engine:setVolume(engine:getVolume() + 0.01)
				end
			else
				if engine:getVolume() > 0 then
					engine:setVolume(engine:getVolume() - 0.01)
				end
			end
		else
			if ship.alive then
				ship.fixture:setRestitution(-1000)
				ship.body:setLinearVelocity(0, 0)
			end
			ship.alive = false
		end
	end
end

function love.draw()

	camera:draw()
	
	love.graphics.setColor(255, 230, 100)
	love.graphics.print(score, 20, 600, 0, 2, 2)
	
	
	
	if math.sqrt( math.pow(hole.body:getX() - ship.body:getX(), 2) + math.pow(hole.body:getY() - ship.body:getY(), 2)) < hole.radius then
		love.graphics.setColor(255, 0, 0)
		love.graphics.print("THE END", size / 2 - 50, size / 2, 0, 2, 2)
		love.graphics.print("press R to restart", size / 2 - 50, size / 2 + 100)
	end
	if pause then
		love.graphics.setColor(0, 0, 0, 100)
		love.graphics.rectangle("fill", 0, 0, size * 2, size * 2)
		
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.draw(fuel, 70, 100, 1.5, 1.5)
		love.graphics.print("increases ships speed", 110, 100, 0, 1.5, 1.5)
		love.graphics.draw(inverser, 70, 150, 1.5, 1.5)
		love.graphics.print("decreases black holes gravitation", 110, 150, 0, 1.5, 1.5)
		love.graphics.draw(coin, 70, 200, 1.5, 1.5)
		love.graphics.print("collect for high score", 110, 200, 0, 1.5, 1.5)
		love.graphics.print("up arrow - boost ship", 150, 300, 0, 2, 2)
		love.graphics.print("left/right arrow - rotate ship", 150, 325, 0, 2, 2)
		love.graphics.print("escape - quit", 150, 350, 0, 2, 2)
		love.graphics.print("+ / - control volume", 150, 375, 0, 2, 2)
		love.graphics.print("space - toggle pause", 50, 500, 0, 4, 4)
		love.graphics.print("(start game!)", 150, 560, 0, 4, 4)
	end
end

function love.keypressed(key)   -- we do not need the unicode, so we can leave it out
	if key == "escape" then
		love.event.push("quit")   -- actually causes the app to quit
	elseif key == " " then
		pause = not pause
		if pause then
			loop:pause()
		else
			loop:play()
		end
	elseif key == "r" and not ship.alive then
		for _, v in ipairs(objects) do
			v.body:destroy()
			v = nil
		end
		objects = {}
		ship.body:setPosition(size / 2, size / 2)
		ship.rotation = 0
		ship.body:setLinearVelocity(0, 0)
		ship.fixture:setRestitution(0.9)
		ship.alive = true
		hole.radius = 20
		gravitation = 2
		speed = 800
		maxSpeed = 200
		score = 0
	elseif (key == "kp+" or key == "+") and love.audio.getVolume() < 1.0 then
		love.audio.setVolume(love.audio.getVolume() + 0.1)
	elseif (key == "kp-" or key == "-") and love.audio.getVolume() > 0.0 then
		love.audio.setVolume(love.audio.getVolume() - 0.1)
	end
end

function applyGravity(body)
	local a = hole.body:getX() - body:getX()
	local b = hole.body:getY() - body:getY()
	
	local x = math.atan2(a,b)
	
	body:applyForce(math.sin(x) * gravitation, math.cos(x) * gravitation)
end

function addItem(x, y)
	local itemObject = items:new()
	itemObject.body = love.physics.newBody(world, 0, 0, "dynamic")
	itemObject.body:setMass(10)
	itemObject.body:setLinearDamping(0.3)
	itemObject.body:setAngle(math.random() * math.random(6))
	sign = math.random(2)
	if sign == 2 then
		sign = -1
	end
	itemObject.body:setAngularVelocity(math.random() * 2 * sign)
	local rnd = math.random(23)
	itemObject:makeThis(rnd)
	itemObject.fixture = love.physics.newFixture(itemObject.body, itemObject.shape, 1)
	itemObject.fixture:setRestitution(0.9)
	itemObject:setDensity(rnd)
	itemObject.body:setPosition(x, y)
	table.insert(objects, itemObject)
end








